"""Perform the construction during the learning process in the background.

"""


import sys
import signal
from enum import Enum
from os import getppid, kill
from queue import Empty
from multiprocessing import Process


class BackgroundJobStatus(Enum):
    DONE = 1


class BackgroundConstructor(Process):

    def __init__(self, input_q, output_q, serial_constructor):
        """

        Args:
            input_q (Queue):
            output_q (Queue):
            serial_constructor (SerialConstructor):
        """
        super(BackgroundConstructor, self).__init__(name="BackgroundConstructor")
        self.input_queue = input_q
        self.output_queue = output_q
        self.serial_constructor = serial_constructor

    def run(self):
        while 1:
            try:
                lb = self.input_queue.get(timeout=0)
                for result in self.serial_constructor.construct(lb):
                    self.output_queue.put(result)
                self.output_queue.put(BackgroundJobStatus.DONE)

            except Empty:
                continue

            except Exception as error:
                parent = getppid()
                kill(parent, signal.SIGALRM)
                sys.exit(-1)
